-- *************
-- * Variables *
-- *************

_Lib = {};
_Lib.path = nil;
_Lib.pName = "WL";





-- ********************
-- * Common functions *
-- ********************

-- Converts argument into a safe for concatenation value.
-- Replaces nil, true or false, table, function with string values. Returns value
function correctValue( v )
   local t = type( v );
   if ( t == "number" or t == "string" ) then
      return v;
   elseif ( t == "boolean" or t == "nil" ) then
      return tostring( v );
   else
      return t;
   end
end

-- Corrects "," and ".". Returns number. Accepts unknown number of arguments.
-- Returns error message, if result is not a number
function getNumber( ... )
   local n = table.concat( arg, "," );
   if ( not tonumber( n ) ) then
      if ( n:find( "%." ) ) then
         n = n:gsub( "%.", "," );
      elseif ( n:find( "," ) ) then
         n = n:gsub( ",", "." );
      end
   end

   n = tonumber( n );
   if ( n ) then
      return n;
   else
      return( "Error. getNumber: result is not a number." );
   end
end





-- **********************
-- * Position functions *
-- **********************

-- Returns coordinates of the top, left, bottom and right
-- borders of the object as an array with named keys
local function getBorders( obj )
   local width, height = obj:GetSize();
   local left, top = obj:GetPosition();
   local b = {
      ["Top"] = top;
      ["Left"] = left;
      ["Bottom"] = top + height;
      ["Right"] = left + width;
   };
   return b;
end

-- Returns object to the visible part of the screen
local function onScreen( obj )
   local left, top = obj:GetPosition();
   local width, height = obj:GetSize();
   local screenWidth, screenHeight = Turbine.UI.Display:GetSize();

   if ( left < 0 ) then
      left = 0;
   end
   if ( top < 0 ) then
      top = 0;
   end

   if ( left >= screenWidth ) then
      left = screenWidth - width;
   end
   if ( top >= screenHeight ) then
      top = screenHeight - height;
   end

   obj:SetPosition( left, top );
end





local function load( partName )
   local file = _Lib.path .. partName;
   import( file );

   _Lib[partName] = lib[partName];

   return lib[partName];
end






-- Interface
--

--[[ Example:

-- Custom library. Start
--
import( "DToX.VitalsM.lib.index" );
local _Lib = DToX.VitalsM.lib._Lib;

_Lib.path = "DToX.VitalsM.lib."; -- For load function
_Lib.pName = "Vitals"; -- For wl in lib file

fonts, colors, align = _Lib.fonts, _Lib.colors, _Lib.align;
getBorders, getPosition = _Lib.getBorders, _Lib.getPosition;
onScreen = _Lib.onScreen;
-- Measure
Text = _Lib.load( "Text" );
-- Chat. Functions: dumpValues, writeLine
_Lib.load( "Chat" );
wl, dv = _Lib.Chat.writeLine, _Lib.Chat.dumpValues;
-- Plugin. Functions: convertData, language/lang, loadData, saveData
Plugin = _Lib.load( "Plugin" );
-- Table. Functions: copy, dump, dupes, getKeys, length/len
Table = _Lib.load( "Table" );
dt = Table.dump;
-- Visual ones
addBorder = _Lib.addBorder;
Hint = _Lib.load( "Hint" ); -- Functions: show, remove
-- Debug/Test ones
_Lib.load( "Debug" ); -- Functions: memoryUsage
mu = _Lib.Debug.memoryUsage;
--
-- Custom library. End

--]]



-- Colors. Default and custom
-- Example: <varName>.Black
local function getColors()
   local c = Turbine.UI.Color;
   c["Grey01"] = c( 0.1, 0.1, 0.1 );
   c["Grey03"] = c( 0.3, 0.3, 0.3 );
   c["Grey05"] = c( 0.5, 0.5, 0.5 );
   c["Grey07"] = c( 0.7, 0.7, 0.7 );
   return c;
end

-- Alignment.
-- Example: <varName>.TopLeft or <varName>[1]
local function getAlign()
   local a = {};
   for k, v in pairs( Turbine.UI.ContentAlignment ) do
      a[k] = v;
      a[v] = v;
   end
   return a;
end

-- Fonts. Example: <varName>.Verdana14
_Lib.fonts = Turbine.UI.Lotro.Font;
_Lib.colors = getColors();
_Lib.align = getAlign();

_Lib.getBorders = getBorders;
_Lib.onScreen = onScreen;

_Lib.load = load;

-- Backward compatibility
-- _Lib.wl = Chat.writeLine;
-- _Lib.dv = Chat.dumpValues;
